-- Healium .9.2.0 - Maintained by Engy of Area 52.  Based on FB Healbox by Dourd of Argent Dawn EU
--
-- Programming notes
-- WARNING In LUA all logical operators consider false and nil as false and anything else as true.  This means not 0 is false!!!!!!!!
-- Color control characters |CAARRGGBB  then |r resets to normal, where AA == Alpha, RR = Red, GG = Green, BB = blue


local Debug = false

Healium = {
  ButtonCount = 5,	-- Current button count (as set by slider)
  Scale = 1.0,
  DoRangeChecks = true,
  RangeCheckPeriod = .5,
  EnableCooldowns = true,
  ShowToolTips = true,
  ShowPercentage = true,
  UseClassColors = false,
  ShowPartyFrame = true,
  ShowPetsFrame = true,
  ShowMeFrame = false,  
  ShowGroupFrames = { },  
  ShowTanks = false,
  ShowBuffs = true
}

-- saved variables per character.  DON'T USE local for these variables.
HealiumDropDown = {}
HealiumDropDownButton = {} -- selected spell names
HealiumDropDownButtonIcon = {} -- selected spell icons

-- Constants
local AddonVersion = "|cFFFFFF00 .9.2.0|r"

local LowHP = 0.6
local VeryLowHP = 0.3
local NamePlateWidth = 120
local HealiumClass = UnitClass("player")
local HealiumRace = UnitRace("player")
local MaxParty = 5 -- Max number of people in party
local MinRangeCheckPeriod = .2 -- .2 = 5Hz
local MaxRangeCheckPeriod = 2  -- 2 = .5Hz
local DefaultRangeCheckPeriod = .5
local MiniMapTexture = "Interface/Icons/Spell_Holy_LayOnHands"  -- Holy Light
local Slash = "/hl" -- the slash command 


-- Global Constants
Healium_MaxButtons = 15		-- Max Possible buttons 
Healium_NamePlateHeight = 28
Healium_xSpacing = 2
Healium_AddonName = "Healium"
Healium_AddonColor = "|cFF55AAFF"

-- Global Variables
Healium_VarsLoaded = nil
Healium_MaxClassSpells = 20 -- For now this is manually set to the max number of class specific spells in Healium_Spell.Name which currently is priest

-- NEW FRAMES VARIABLES
Healium_Units = { { } } -- table of tables that maps unit names to their frame, used for efficient handling of UNIT_HEALTH so each button doesn't get a UNIT_HEALTH event for every unit.
Healium_Frames = { } -- table of all created "unit" frames.  Can access buttons from each of these.

Healium_ButtonIDs = { } -- table of IDs that correspond to the selected spells, not persisted 


Healium_Spell = {		-- List of spells, icons for the spells, and IDs
  Name = {},
  Icon = {},
  ID = {}
}

local ClassIcon = {
        Druid = "Interface/Icons/INV_Misc_MonsterClaw_04",
        Warlock = "Interface/Icons/Spell_Nature_FaerieFire",
        Hunter = "Interface/Icons/INV_Weapon_Bow_07",
        Mage = "Interface/Icons/INV_Staff_13",
        Priest = "Interface/Icons/INV_Staff_30",
        Warrior = "Interface/Icons/INV_Sword_27",
        Shaman = "Interface/Icons/Spell_Nature_BloodLust",
        Paladin = "Interface/Icons/Ability_ThunderBolt",
        Rogue = "Interface/AddOns/ChatIcons/images/UI-CharacterCreate-Classes_Rogue",
		["Death Knight"] = "Interface/Icons/Spell_Deathknight_ClassIcon"
}

Healium_InitSpells(HealiumClass, HealiumRace)

function Healium_Print(msg)
	DEFAULT_CHAT_FRAME:AddMessage(Healium_AddonColor .. Healium_AddonName .. "|r " .. tostring(msg))		
end

function Healium_DebugPrint(msg)
	if (Debug) then
		Healium_Print("Debug: " .. tostring(msg))		
	end
end

function Healium_Warn(msg)
	DEFAULT_CHAT_FRAME:AddMessage("|CFFFF0000Warning|r: " .. tostring(msg))		
end

function Healium_OnLoad(self)
 	Healium_Print(AddonVersion.." |cFF00FF00Loaded |rClick The MiniMap Button For Options")
 
	self:RegisterEvent("ADDON_LOADED")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("SPELLS_CHANGED")
	self:RegisterEvent("UNIT_HEALTH")
	self:RegisterEvent("VARIABLES_LOADED")
	self:RegisterEvent("UNIT_SPELLCAST_SENT")	
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	self:RegisterEvent("PLAYER_TALENT_UPDATE")
	self:RegisterEvent("UNIT_AURA")
	self:RegisterEvent("SPELL_UPDATE_COOLDOWN")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	
end

local function UpdatePercentageVisibility()
	for _, k in ipairs(Healium_Frames) do
		if Healium.ShowPercentage then
			k.HPText:Show()
		else
			k.HPText:Hide()
		end
	end
end

-- Sets the health bar color based on the unit's health ONLY
local function UpdateHealthBar(HPPercent, frame)
	if (HPPercent > LowHP) then 
		frame.HealthBar:SetStatusBarColor(0,1,0,1) 
	end
	if (HPPercent < LowHP) then 
		frame.HealthBar:SetStatusBarColor(1,0.9,0,1) 
	end
	if (HPPercent < VeryLowHP) then
		frame.HealthBar:SetStatusBarColor(1,0,0,1) 
	end
end

local function UpdateClassColors()
	for _, k in ipairs(Healium_Frames) do
		if (k.TargetUnit) then
			if not UnitExists(k.TargetUnit) then return end
			if Healium.UseClassColors then
				local class = select(2, UnitClass(k.TargetUnit)) or "WARRIOR"
				local color = RAID_CLASS_COLORS[class]
				k.HealthBar:SetStatusBarColor(color.r, color.g, color.b)				
			else
				local Health = UnitHealth(k.TargetUnit)
				local MaxHealth = UnitHealthMax(k.TargetUnit)
				HPPercent =  Health / MaxHealth
				UpdateHealthBar(HPPercent, k)
			end
		end
	end
end

function Healium_UpdateUnitHealth(UnitName, NamePlate)
	if not NamePlate then return end
	if not UnitExists(UnitName) then return end
	
	local Health = UnitHealth(UnitName)
	local MaxHealth = UnitHealthMax(UnitName)
		
	 HPPercent =  Health / MaxHealth
	 if HPPercent > 1 then 
		HPPercent = 1
	 end
	 NamePlate.HPText:SetText( format("%.1i%%", HPPercent*100))
	 NamePlate.HealthBar:SetMinMaxValues(0,MaxHealth)
	 NamePlate.HealthBar:SetValue(Health)
	 
	if Healium.UseClassColors then
		local class = select(2, UnitClass(UnitName)) or "WARRIOR"
		local color = RAID_CLASS_COLORS[class]
		NamePlate.HealthBar:SetStatusBarColor(color.r, color.g, color.b)					
	else
		UpdateHealthBar(HPPercent, NamePlate)
	end
end

local function GetSpellID(spell)
    local i = 1
    local spellID
    local highestRank
    while true do
        local spellName = GetSpellName(i, SpellBookFrame.bookType)
        if (not spellName) then
            break
        end
        if (spellName == spell) then
            spellID = i
            highestRank = spellRank
        end
        i = i + 1
        if (i > 300) then
            break
        end
    end            
    return spellID, highestRank
end

-- Warning UpdateSpells() is a global function from Blizzard. 
-- Loops through Healium_Spell.Name[] and updates it's corresponding Healium_Spells.ID[] 
function Healium_UpdateSpells()
	for k, v in ipairs (Healium_Spell.Name) do
		Healium_Spell.ID[k] = GetSpellID(Healium_Spell.Name[k])
		if (Healium_Spell.ID[k]) then
			Healium_Spell.Icon[k]= GetSpellTexture(Healium_Spell.ID[k], BOOKTYPE_SPELL)
--[[
			if (Debug) then 
				toPrint = "ID: "..Healium_Spell.ID[k].." | Name: "..Healium_Spell.Name[k].." | Texture: "..Healium_Spell.Icon[k]
				Healium_Print(toPrint)
			end
--]]
		end
	end 

	Healium_UpdateButtonSpells()
end


-- Efficient cooldowns
local function Healium_UpdateButtonCooldownsByColumn(column)
	if Healium_ButtonIDs[column] then
		local start, duration, enable = GetSpellCooldown(Healium_ButtonIDs[column], SpellBookFrame.bookType)
		
		for _,j in pairs(Healium_Units) do
			for x,y in pairs(j) do
				local button = y.buttons[column]
				if button then 
					if button:IsShown() then 
						CooldownFrame_SetTimer(button.cooldown, start, duration, enable)
					end
				end
			end
		end
	end
end

local function Healium_UpdateButtonCooldowns()
	for i=1, Healium.ButtonCount, 1 do
		Healium_UpdateButtonCooldownsByColumn(i)
	end
end


function Healium_HealButton_OnLoad(self)
	self.TimeSinceLastUpdate = 0
	self:RegisterEvent("SPELL_UPDATE_USABLE")
	self:RegisterForDrag("LeftButton")
end

function Healium_HealButton_OnUpdate(self, elapsed)
--	if ( (not HealiumActive) or (not Healium.DoRangeChecks) ) then return 0 end 
	if ( not Healium.DoRangeChecks ) then return 0 end 	
	self.TimeSinceLastUpdate = self.TimeSinceLastUpdate + elapsed 	

	if (self.TimeSinceLastUpdate > Healium.RangeCheckPeriod) then
		Healium_RangeCheckButton(self)
		self.TimeSinceLastUpdate = 0
	end
end

function Healium_HealButton_OnEnter(frame, motion)
    GameTooltip:SetOwner(frame, "ANCHOR_RIGHT", -30, 5)
    if (frame.id) then
		if (not Healium.ShowToolTips) then return end	
		GameTooltip_SetDefaultAnchor(GameTooltip, frame)
		GameTooltip:SetSpell(frame.id, SpellBookFrame.bookType)
		local unit = frame:GetParent().TargetUnit
		if not UnitExists(unit) then return end
		local Name = UnitName(unit)
		if (not Name) then Name = "-" end
        GameTooltip:AddLine("Target: |cFF00FF00"..Name,1,1,1)

--		if (Debug) then 
--			local texture = frame.icon:GetTexture() or "(nil)"
--			GameTooltip:AddLine("ID= ".. frame.id .. "|nTexture=" .. texture) 
--		end
--        GameTooltip:AddLine(Healium_AddonColor.. "---- " .. Healium_AddonName .. " button ----",1,1,1)		  
		GameTooltip:Show()		
	else
		-- Safely Handle Empty Buttons	
		GameTooltip:SetText("|cFFFFFFFFNo Spell|n|cFF00FF00Please go to Interface, Addons, " ..Healium_AddonName .. "|nand select your spells from the list.")
		GameTooltip:Show()		
    end
end

function Healium_HealButton_OnLeave()
	GameTooltip:Hide()
end

function Healium_HealButton_OnEvent(self, event)
	if (not self.id) then return 0 end   
	
	if event == "SPELL_UPDATE_USABLE" then
		Healium_RangeCheckButton(self)
	end
end

function Healium_HealButton_OnReceiveDrag(self)
	if CursorHasSpell() then
		local infoType, info1, info2 = GetCursorInfo()
		if infoType == "spell" then 
			if InCombatLockdown() then
				Healium_Warn("Can't update button while in combat")
				return
			end
			
			if (self.index > 0) and (self.index <= Healium_MaxButtons) then
				local spellName = GetSpellName(info1, BOOKTYPE_SPELL )		
				if IsPassiveSpell(info1, BOOKTYPE_SPELL) then
					local link = GetSpellLink(info1, BOOKTYPE_SPELL)
					Healium_Warn(link .. " is a passive spell and cannot be used in " .. Healium_AddonName)
					return
				end
				local name, rank, icon = GetSpellInfo(spellName)
				HealiumDropDownButton[self.index] = name
				HealiumDropDownButtonIcon[self.index] = icon
				UIDropDownMenu_SetText(HealiumDropDown[self.index], HealiumDropDownButton[self.index])
				
				Healium_UpdateButtonSpells()
				Healium_UpdateButtonIcons()				
				Healium_UpdateButtonCooldownsByColumn(self.index)	
				ClearCursor()
			end
		end
	end
end

-- [ DropDown Menu Creation Function ] --------------------------------------------------

function Healium_CreateBoxDropDownMenu(name,parent,x,y)

  local f = CreateFrame("Frame", name, parent, "UIDropDownMenuTemplate") 
  f:SetPoint("TOPLEFT", parent, "TOPLEFT",x,y)
  UIDropDownMenu_SetWidth(f, 180)  

  
  f.Text = f:CreateFontString(nil, "OVERLAY","GameFontNormal")
  f.Text:SetPoint("TOPLEFT",f,"TOPLEFT",-50,-5)
  
  return f

end

local function DropDownMenu_Init(self,level)

  level = level or 1  
  local info = UIDropDownMenu_CreateInfo() 

  for k, v in ipairs (Healium_Spell.Name) do
    info.text = Healium_Spell.Name[k] 
    info.value = k-1
    info.func = Healium_DropDownMenuItem_OnClick
    info.owner = this:GetParent() 
    info.checked = nil 
    info.icon = Healium_Spell.Icon[k]
    if (info.icon) then UIDropDownMenu_AddButton(info, level) end
  end
end

-- [ DropDown Menu Interaction Function ] --------------------------------------------------

function Healium_DropDownMenuItem_OnClick(dropdownbutton)
  UIDropDownMenu_SetSelectedValue(dropdownbutton.owner, dropdownbutton.value) 

	for i=1, Healium_MaxClassSpells, 1 do
		if (dropdownbutton.owner == HealiumDropDown[i]) then
			for j=0, Healium_MaxClassSpells - 1, 1 do
				if (dropdownbutton.value == j) then
					HealiumDropDownButton[i] = Healium_Spell.Name[j+1]
					HealiumDropDownButtonIcon[i] = Healium_Spell.Icon[j+1]
				end
			end
		end
	end
	Healium_UpdateButtonIcons()
	Healium_UpdateButtonSpells()	
end

local function UpdateRangeCheckSliderText()
    Healium_RangeCheckSlider.Text:SetText("Range Check Frequency: |cFFFFFFFF".. format("%.1f",Healium_RangeCheckSlider:GetValue()) .. " Hz")
end

function Healium_CreateAddonOptionFrame()
--	Healium_DebugPrint("Begin Healium_CreateAddonOptionFrame()")
	
	local panel = CreateFrame("Frame", nil, UIParent)
	panel.name = Healium_AddonName
	panel.okay = function (self)self.originalValue = MY_VARIABLE end    -- [[ When the player clicks okay, set the original value to the current setting ]] --
	panel.cancel = function (self) MY_VARIABLE = self.originalValue end    -- [[ When the player clicks cancel, set the current setting to the original value ]] --
	InterfaceOptions_AddCategory(panel)

	local scrollframe = CreateFrame("ScrollFrame", "HealiumPanelScroller", panel, "UIPanelScrollFrameTemplate") 
	local framewidth = InterfaceOptionsFramePanelContainer:GetWidth()
	local frameheight = InterfaceOptionsFramePanelContainer:GetHeight() 
	scrollframe:SetPoint("TOPLEFT", panel, "TOPLEFT", 10, -25)
	scrollframe:SetWidth(framewidth-45)
	scrollframe:SetHeight(frameheight-45)
	scrollframe:Show()
	
	local scrollchild = CreateFrame("Frame", nil, UIParent)
	
	scrollchild:SetHeight(frameheight-45)
	scrollchild:SetWidth(framewidth-45)
	scrollframe:SetScrollChild(scrollchild)	
	scrollchild:Show()

	
	-- Title text
	local TitleText = scrollchild:CreateFontString(nil, "OVERLAY","GameFontNormalLarge")
	TitleText:SetJustifyH("LEFT")
	TitleText:SetPoint("TOPLEFT", 10, -10)
	TitleText:SetText(Healium_AddonColor .. Healium_AddonName .. "|r" .. AddonVersion)

	-- Title subtext
	local TitleSubText = scrollchild:CreateFontString(nil, "OVERLAY","GameFontNormalSmall")
	TitleSubText:SetJustifyH("LEFT")
	TitleSubText:SetPoint("TOPLEFT", 10, -30)
	TitleSubText:SetText("Welcome to the " .. Healium_AddonName.. "  options screen.|nUse the scrollbar to access more options.")
	TitleSubText:SetTextColor(1,1,1,1) 
  
	-- Create the Class Icon 
  	local HealiumClassIcon = CreateFrame("Frame", HealiumClassIcon,scrollchild)
	HealiumClassIcon:SetPoint("TOPRIGHT",-20,0)
	HealiumClassIconTexture = HealiumClassIcon:CreateTexture(nil, "BACKGROUND")
	HealiumClassIconTexture:SetAllPoints()
	HealiumClassIconTexture:SetTexture(ClassIcon[HealiumClass])
	HealiumClassIcon:SetHeight(60)
	HealiumClassIcon:SetWidth(60)
	HealiumClassIcon.Text = HealiumClassIcon:CreateFontString(nil, "OVERLAY","GameFontNormalLarge")
	HealiumClassIcon.Text:SetText(strupper(HealiumClass))
	HealiumClassIcon.Text:SetPoint("CENTER",0,-38)
	HealiumClassIcon.Text:SetTextColor(1,1,0.2,1)
 	
	-- ToolTips Check Button
    Healium_TooltipsCheck = CreateFrame("CheckButton","$parentShowTooltipCheckButton",scrollchild,"OptionsCheckButtonTemplate")
	Healium_TooltipsCheck:SetPoint("TOPLEFT",5,-70)	
    
    Healium_TooltipsCheck.Text = Healium_TooltipsCheck:CreateFontString(nil, "BACKGROUND","GameFontNormal")
	Healium_TooltipsCheck.Text:SetPoint("LEFT", Healium_TooltipsCheck, "RIGHT", 0)
    Healium_TooltipsCheck.Text:SetText("Show Button ToolTips")
    
    Healium_TooltipsCheck:SetScript("OnClick",function()
        Healium.ShowToolTips = Healium_TooltipsCheck:GetChecked() or false
    end)	
	
	-- Percentage Check button
    Healium_PercentageCheck = CreateFrame("CheckButton","$parentShowTooltipCheckButton",scrollchild,"OptionsCheckButtonTemplate")
    Healium_PercentageCheck:SetPoint("TOPLEFT", Healium_TooltipsCheck, "BOTTOMLEFT", 0, 0)
    
    Healium_PercentageCheck.Text = Healium_TooltipsCheck:CreateFontString(nil, "BACKGROUND","GameFontNormal")
	Healium_PercentageCheck.Text:SetPoint("LEFT", Healium_PercentageCheck, "RIGHT", 0)
    Healium_PercentageCheck.Text:SetText("Show Health Percentage")
    
    Healium_PercentageCheck:SetScript("OnClick",function()
        Healium.ShowPercentage = Healium_PercentageCheck:GetChecked() or false
		UpdatePercentageVisibility()
    end)	
	
	-- ClassColor Check button
    Healium_ClassColorCheck = CreateFrame("CheckButton","$parentShowTooltipCheckButton",scrollchild,"OptionsCheckButtonTemplate")
    Healium_ClassColorCheck:SetPoint("TOPLEFT", Healium_PercentageCheck, "BOTTOMLEFT", 0, 0)
    
    Healium_ClassColorCheck.Text = Healium_TooltipsCheck:CreateFontString(nil, "BACKGROUND","GameFontNormal")
	Healium_ClassColorCheck.Text:SetPoint("LEFT", Healium_ClassColorCheck, "RIGHT", 0)
    Healium_ClassColorCheck.Text:SetText("Use Class Colors")
    
    Healium_ClassColorCheck:SetScript("OnClick",function()
        Healium.UseClassColors = Healium_ClassColorCheck:GetChecked() or false
		UpdateClassColors()
    end)	

	
 	-- Dropdown menus
	local ButtonConfigTitleText = scrollchild:CreateFontString(nil, "OVERLAY","GameFontNormalLarge")
	ButtonConfigTitleText:SetJustifyH("LEFT")
	ButtonConfigTitleText:SetPoint("TOPLEFT", Healium_ClassColorCheck, "BOTTOMLEFT", 0, -30)
	ButtonConfigTitleText:SetText("Button Configuration")	
	
	local ButtonConfigTitleSubText = scrollchild:CreateFontString(nil, "OVERLAY","GameFontNormalSmall")
	ButtonConfigTitleSubText:SetJustifyH("LEFT")
	ButtonConfigTitleSubText:SetPoint("TOPLEFT", ButtonConfigTitleText, "BOTTOMLEFT", 0, 0)
	ButtonConfigTitleSubText:SetText("Click the dropdowns to configure each button.|nYou may now drag and drop directly from the spellbook|nonto buttons to configure them, including buffs!")
	ButtonConfigTitleSubText:SetTextColor(1,1,1,1) 	
	
	y = -250
	y_inc = 20
	for i=1, Healium_MaxButtons, 1 do
		HealiumDropDown[i] = Healium_CreateBoxDropDownMenu("HealiumDropDown[" .. i .. "]",scrollchild,60,y)
		y = y - y_inc
		HealiumDropDown[i].Text:SetText("Button " .. i)
	end

	-- Slider for controling how many buttons to show
	if (MaxButtonSlider) then MaxButtonSlider:Hide() end

    MaxButtonSlider = CreateFrame("Slider","MaxButtonSlider",scrollchild,"OptionsSliderTemplate")
    MaxButtonSlider:SetWidth(128)
    MaxButtonSlider:SetHeight(16)
    
    local t = MaxButtonSlider:CreateTexture(nil,"BACKGROUND")
    t:SetAllPoints(f)
    MaxButtonSlider.texture = t
      
    MaxButtonSlider:SetPoint("TOPLEFT", 220, -110)
      
    MaxButtonSlider:SetMinMaxValues(0,Healium_MaxButtons)
    MaxButtonSlider:SetValueStep(1)
    MaxButtonSlider:SetValue(Healium.ButtonCount)
      
    MaxButtonSlider.Text = MaxButtonSlider:CreateFontString(nil, "BACKGROUND","GameFontNormalLarge")
    MaxButtonSlider.Text:SetPoint("CENTER", 0, 17)
    MaxButtonSlider.Text:SetText("Show |cFFFFFFFF"..MaxButtonSlider:GetValue().. "|r Buttons")
      
    getglobal(MaxButtonSlider:GetName().."Low"):SetText("0")
    getglobal(MaxButtonSlider:GetName().."High"):SetText(Healium_MaxButtons)
      
    MaxButtonSlider:SetScript("OnValueChanged",MaxButtonSlider_Update)
    MaxButtonSlider:Show()
  
    -- Slider for Scaling
    Healium_ScaleSlider = CreateFrame("Slider","HealiumScaleSlider",scrollchild,"OptionsSliderTemplate")
    Healium_ScaleSlider:SetWidth(100)
    Healium_ScaleSlider:SetHeight(16)
    
    Healium_ScaleSlider.texture = t
    
    getglobal(Healium_ScaleSlider:GetName().."Low"):SetText("Small")
    getglobal(Healium_ScaleSlider:GetName().."High"):SetText("Large")
    
    Healium_ScaleSlider:SetMinMaxValues(0.6,1.5)
    Healium_ScaleSlider:SetValueStep(0.1)
    Healium_ScaleSlider:SetValue(Healium.Scale)
    
    Healium_ScaleSlider:SetPoint("TOPLEFT", MaxButtonSlider, "BOTTOMLEFT", 0, -30)
    
    Healium_ScaleSlider.Text = Healium_ScaleSlider:CreateFontString(nil, "BACKGROUND","GameFontNormalLarge")
    Healium_ScaleSlider.Text:SetPoint("CENTER", -5, 17)
    Healium_ScaleSlider.Text:SetText("Scale: |cFFFFFFFF".. format("%.1f",Healium_ScaleSlider:GetValue()))
    
    Healium_ScaleSlider:SetScript("OnValueChanged",function()
        Healium.Scale = Healium_ScaleSlider:GetValue()
		Healium_SetScale()
        Healium_ScaleSlider.Text:SetText("Scale: |cFFFFFFFF".. format("%.1f",Healium.Scale))
    end)	
	
	-- Show Frames Settings
	local ShowFramesTitleText = scrollchild:CreateFontString(nil, "OVERLAY","GameFontNormalLarge")
	ShowFramesTitleText:SetJustifyH("LEFT")
	ShowFramesTitleText:SetPoint("TOPLEFT", HealiumDropDown[Healium_MaxButtons].Text, "BOTTOMLEFT", 0, -30)
	ShowFramesTitleText:SetText("Show Frames")	
	
	local ShowFramesTitleSubText = scrollchild:CreateFontString(nil, "OVERLAY","GameFontNormalSmall")
	ShowFramesTitleSubText:SetJustifyH("LEFT")
	ShowFramesTitleSubText:SetPoint("TOPLEFT", ShowFramesTitleText, "BOTTOMLEFT", 0, 0)
	ShowFramesTitleSubText:SetText("Check each frame to show.")
	ShowFramesTitleSubText:SetTextColor(1,1,1,1) 
	
	-- Show Party Check
    Healium_ShowPartyCheck = CreateFrame("CheckButton","$parentShowPartyCheckButton",scrollchild,"OptionsCheckButtonTemplate")
    Healium_ShowPartyCheck:SetPoint("TOPLEFT",ShowFramesTitleSubText, "BOTTOMLEFT", 0, -10)
    Healium_ShowPartyCheck.Text = Healium_ShowPartyCheck:CreateFontString(nil, "BACKGROUND","GameFontNormal")
    Healium_ShowPartyCheck.Text:SetPoint("LEFT", Healium_ShowPartyCheck, "RIGHT", 0)
    Healium_ShowPartyCheck.Text:SetText("Party")
    
    Healium_ShowPartyCheck:SetScript("OnClick",function()
        Healium.ShowPartyFrame = Healium_ShowPartyCheck:GetChecked() or false
		Healium_ShowHidePartyFrame()
    end)

	-- Show Pets Check
    Healium_ShowPetsCheck = CreateFrame("CheckButton","$parentShowPetsCheckButton",scrollchild,"OptionsCheckButtonTemplate")
    Healium_ShowPetsCheck:SetPoint("TOPLEFT",Healium_ShowPartyCheck, "BOTTOMLEFT", 0, 0)
    Healium_ShowPetsCheck.Text = Healium_ShowPetsCheck:CreateFontString(nil, "BACKGROUND","GameFontNormal")
    Healium_ShowPetsCheck.Text:SetPoint("LEFT", Healium_ShowPetsCheck, "RIGHT", 0)
    Healium_ShowPetsCheck.Text:SetText("Pets")
    
    Healium_ShowPetsCheck:SetScript("OnClick",function()
        Healium.ShowPetsFrame = Healium_ShowPetsCheck:GetChecked() or false
		Healium_ShowHidePetsFrame()
    end)

	-- Show Me Check
    Healium_ShowMeCheck = CreateFrame("CheckButton","$parentShowMeCheckButton",scrollchild,"OptionsCheckButtonTemplate")
    Healium_ShowMeCheck:SetPoint("TOPLEFT",Healium_ShowPetsCheck, "BOTTOMLEFT", 0, 0)
    Healium_ShowMeCheck.Text = Healium_ShowMeCheck:CreateFontString(nil, "BACKGROUND","GameFontNormal")
    Healium_ShowMeCheck.Text:SetPoint("LEFT", Healium_ShowMeCheck, "RIGHT", 0)
    Healium_ShowMeCheck.Text:SetText("Me")
    
    Healium_ShowMeCheck:SetScript("OnClick",function()
        Healium.ShowMeFrame = Healium_ShowMeCheck:GetChecked() or false
		Healium_ShowHideMeFrame()
    end)
	
	-- Show Group 1 Check
    Healium_ShowGroup1Check = CreateFrame("CheckButton","$parentShowGroup1CheckButton",scrollchild,"OptionsCheckButtonTemplate")
    Healium_ShowGroup1Check:SetPoint("TOPLEFT",Healium_ShowMeCheck, "BOTTOMLEFT", 0, 0)
    Healium_ShowGroup1Check.Text = Healium_ShowGroup1Check:CreateFontString(nil, "BACKGROUND","GameFontNormal")
    Healium_ShowGroup1Check.Text:SetPoint("LEFT", Healium_ShowGroup1Check, "RIGHT", 0)
    Healium_ShowGroup1Check.Text:SetText("Group 1")
    
    Healium_ShowGroup1Check:SetScript("OnClick",function()
        Healium.ShowGroupFrames[1] = Healium_ShowGroup1Check:GetChecked() or false
		Healium_ShowHideGroupFrame(1)		
    end)	
	
	-- Show Group 2 Check
    Healium_ShowGroup2Check = CreateFrame("CheckButton","$parentShowGroup2CheckButton",scrollchild,"OptionsCheckButtonTemplate")
    Healium_ShowGroup2Check:SetPoint("TOPLEFT",Healium_ShowGroup1Check, "BOTTOMLEFT", 0, 0)
    Healium_ShowGroup2Check.Text = Healium_ShowGroup2Check:CreateFontString(nil, "BACKGROUND","GameFontNormal")
    Healium_ShowGroup2Check.Text:SetPoint("LEFT", Healium_ShowGroup2Check, "RIGHT", 0)
    Healium_ShowGroup2Check.Text:SetText("Group 2")
    
    Healium_ShowGroup2Check:SetScript("OnClick",function()
        Healium.ShowGroupFrames[2] = Healium_ShowGroup2Check:GetChecked() or false
		Healium_ShowHideGroupFrame(2)
    end)		
	
	-- Show Group 3 Check
    Healium_ShowGroup3Check = CreateFrame("CheckButton","$parentShowGroup3CheckButton",scrollchild,"OptionsCheckButtonTemplate")
    Healium_ShowGroup3Check:SetPoint("TOPLEFT",Healium_ShowGroup2Check, "BOTTOMLEFT", 0, 0)
    Healium_ShowGroup3Check.Text = Healium_ShowGroup3Check:CreateFontString(nil, "BACKGROUND","GameFontNormal")
    Healium_ShowGroup3Check.Text:SetPoint("LEFT", Healium_ShowGroup3Check, "RIGHT", 0)
    Healium_ShowGroup3Check.Text:SetText("Group 3")
    
    Healium_ShowGroup3Check:SetScript("OnClick",function()
        Healium.ShowGroupFrames[3] = Healium_ShowGroup3Check:GetChecked() or false
		Healium_ShowHideGroupFrame(3)		
    end)		

	-- Show Group 4 Check
    Healium_ShowGroup4Check = CreateFrame("CheckButton","$parentShowGroup4CheckButton",scrollchild,"OptionsCheckButtonTemplate")
    Healium_ShowGroup4Check:SetPoint("TOPLEFT",Healium_ShowGroup3Check, "BOTTOMLEFT", 0, 0)
    Healium_ShowGroup4Check.Text = Healium_ShowGroup4Check:CreateFontString(nil, "BACKGROUND","GameFontNormal")
    Healium_ShowGroup4Check.Text:SetPoint("LEFT", Healium_ShowGroup4Check, "RIGHT", 0)
    Healium_ShowGroup4Check.Text:SetText("Group 4")
    
    Healium_ShowGroup4Check:SetScript("OnClick",function()
        Healium.ShowGroupFrames[4]= Healium_ShowGroup4Check:GetChecked() or false
		Healium_ShowHideGroupFrame(4)		
    end)			
	
	-- Show Group 5 Check
    Healium_ShowGroup5Check = CreateFrame("CheckButton","$parentShowGroup5CheckButton",scrollchild,"OptionsCheckButtonTemplate")
    Healium_ShowGroup5Check:SetPoint("TOPLEFT",Healium_ShowGroup4Check, "BOTTOMLEFT", 0, 0)
    Healium_ShowGroup5Check.Text = Healium_ShowGroup5Check:CreateFontString(nil, "BACKGROUND","GameFontNormal")
    Healium_ShowGroup5Check.Text:SetPoint("LEFT", Healium_ShowGroup5Check, "RIGHT", 0)
    Healium_ShowGroup5Check.Text:SetText("Group 5")
    
    Healium_ShowGroup5Check:SetScript("OnClick",function()
        Healium.ShowGroupFrames[5] = Healium_ShowGroup5Check:GetChecked() or false
		Healium_ShowHideGroupFrame(5)
    end)		

	-- Show Group 6 Check
    Healium_ShowGroup6Check = CreateFrame("CheckButton","$parentShowGroup6CheckButton",scrollchild,"OptionsCheckButtonTemplate")
    Healium_ShowGroup6Check:SetPoint("TOPLEFT",Healium_ShowGroup5Check, "BOTTOMLEFT", 0, 0)
    Healium_ShowGroup6Check.Text = Healium_ShowGroup6Check:CreateFontString(nil, "BACKGROUND","GameFontNormal")
    Healium_ShowGroup6Check.Text:SetPoint("LEFT", Healium_ShowGroup6Check, "RIGHT", 0)
    Healium_ShowGroup6Check.Text:SetText("Group 6")
    
    Healium_ShowGroup6Check:SetScript("OnClick",function()
        Healium.ShowGroupFrames[6] = Healium_ShowGroup6Check:GetChecked() or false
		Healium_ShowHideGroupFrame(6)
    end)	
	
	-- Show Group 7 Check
    Healium_ShowGroup7Check = CreateFrame("CheckButton","$parentShowGroup7CheckButton",scrollchild,"OptionsCheckButtonTemplate")
    Healium_ShowGroup7Check:SetPoint("TOPLEFT",Healium_ShowGroup6Check, "BOTTOMLEFT", 0, 0)
    Healium_ShowGroup7Check.Text = Healium_ShowGroup7Check:CreateFontString(nil, "BACKGROUND","GameFontNormal")
    Healium_ShowGroup7Check.Text:SetPoint("LEFT", Healium_ShowGroup7Check, "RIGHT", 0)
    Healium_ShowGroup7Check.Text:SetText("Group 7")
    
    Healium_ShowGroup7Check:SetScript("OnClick",function()
        Healium.ShowGroupFrames[7] = Healium_ShowGroup7Check:GetChecked() or false
		Healium_ShowHideGroupFrame(7)		
    end)	
	
	-- Show Group 8 Check
    Healium_ShowGroup8Check = CreateFrame("CheckButton","$parentShowGroup8CheckButton",scrollchild,"OptionsCheckButtonTemplate")
    Healium_ShowGroup8Check:SetPoint("TOPLEFT",Healium_ShowGroup7Check, "BOTTOMLEFT", 0, 0)
    Healium_ShowGroup8Check.Text = Healium_ShowGroup8Check:CreateFontString(nil, "BACKGROUND","GameFontNormal")
    Healium_ShowGroup8Check.Text:SetPoint("LEFT", Healium_ShowGroup8Check, "RIGHT", 0)
    Healium_ShowGroup8Check.Text:SetText("Group 8")
    
    Healium_ShowGroup8Check:SetScript("OnClick",function()
        Healium.ShowGroupFrames[8] = Healium_ShowGroup8Check:GetChecked() or false
		Healium_ShowHideGroupFrame(8)
    end)		
	
    Healium_ShowTanksCheck = CreateFrame("CheckButton","$parentShowTanksCheckButton",scrollchild,"OptionsCheckButtonTemplate")
    Healium_ShowTanksCheck:SetPoint("TOPLEFT",Healium_ShowGroup8Check, "BOTTOMLEFT", 0, 0)
    Healium_ShowTanksCheck.Text = Healium_ShowTanksCheck:CreateFontString(nil, "BACKGROUND","GameFontNormal")
    Healium_ShowTanksCheck.Text:SetPoint("LEFT", Healium_ShowTanksCheck, "RIGHT", 0)
    Healium_ShowTanksCheck.Text:SetText("Tanks")
    
    Healium_ShowTanksCheck:SetScript("OnClick",function()
        Healium.ShowTanksFrame = Healium_ShowTanksCheck:GetChecked() or false
		Healium_ShowHideTanksFrame()
    end)			
	
	-- CPU Intensive Settings text
	local UpdatingTitleText = scrollchild:CreateFontString(nil, "OVERLAY","GameFontNormalLarge")
	UpdatingTitleText:SetJustifyH("LEFT")
	UpdatingTitleText:SetPoint("TOPLEFT", Healium_ShowTanksCheck, "BOTTOMLEFT", 0, -30)
	UpdatingTitleText:SetText("CPU Intensive Settings")

	local UpdatingTitleSubText = scrollchild:CreateFontString(nil, "OVERLAY","GameFontNormalSmall")
	UpdatingTitleSubText:SetJustifyH("LEFT")
	UpdatingTitleSubText:SetPoint("TOPLEFT", UpdatingTitleText, "BOTTOMLEFT", 0, 0)
	UpdatingTitleSubText:SetText("Enabling these settings may cause extra lag.")
	UpdatingTitleSubText:SetTextColor(1,1,1,1) 
	
    -- EnableColldowns Check Button
    Healium_EnableCooldownsCheck = CreateFrame("CheckButton","$parentEnableCooldownsCheckButton",scrollchild,"OptionsCheckButtonTemplate")
    Healium_EnableCooldownsCheck:SetPoint("TOPLEFT", UpdatingTitleSubText, "BOTTOMLEFT", 0, -10)
    
    Healium_EnableCooldownsCheck.Text = Healium_EnableCooldownsCheck:CreateFontString(nil, "BACKGROUND","GameFontNormal")
    Healium_EnableCooldownsCheck.Text:SetPoint("LEFT", Healium_EnableCooldownsCheck, "RIGHT", 0)
    Healium_EnableCooldownsCheck.Text:SetText("Enable Cooldowns")
    
    Healium_EnableCooldownsCheck:SetScript("OnClick",function()
        Healium.EnableCooldowns = Healium_EnableCooldownsCheck:GetChecked() or false
    end)	
	

	-- RangeCheck Check Button
    Healium_RangeCheckCheck = CreateFrame("CheckButton","$parentRangeCheckButton",scrollchild,"OptionsCheckButtonTemplate")
    Healium_RangeCheckCheck:SetPoint("TOPLEFT",Healium_EnableCooldownsCheck, "BOTTOMLEFT", 0, 0)
    
    Healium_RangeCheckCheck.Text = Healium_RangeCheckCheck:CreateFontString(nil, "BACKGROUND","GameFontNormal")
    Healium_RangeCheckCheck.Text:SetPoint("LEFT", Healium_RangeCheckCheck, "RIGHT", 0)
    Healium_RangeCheckCheck.Text:SetText("Enable Range Checks")
    
    Healium_RangeCheckCheck:SetScript("OnClick",function()
        Healium.DoRangeChecks = Healium_RangeCheckCheck:GetChecked() or false
--		Healium_UpdateNames()
    end)
	
	-- RangeCheck Slider
	Healium_RangeCheckSlider = CreateFrame("Slider","$parentRangeCheckSlider",scrollchild,"OptionsSliderTemplate")
    Healium_RangeCheckSlider:SetWidth(180)
    Healium_RangeCheckSlider:SetHeight(16)
    
    Healium_RangeCheckSlider.texture = t
    
    getglobal(Healium_RangeCheckSlider:GetName().."Low"):SetText("Slower\n(Less CPU)")
    getglobal(Healium_RangeCheckSlider:GetName().."High"):SetText("Faster\n(More CPU)")
    
    Healium_RangeCheckSlider:SetMinMaxValues(.5,5.0)
    Healium_RangeCheckSlider:SetValueStep(0.1)
    Healium_RangeCheckSlider:SetValue(1.0/Healium.RangeCheckPeriod)
    
    Healium_RangeCheckSlider:SetPoint("TOPLEFT", Healium_RangeCheckCheck.Text, "TOPRIGHT", 15, 0)
    
    Healium_RangeCheckSlider.Text = Healium_RangeCheckSlider:CreateFontString(nil, "BACKGROUND","GameFontNormalSmall")
    Healium_RangeCheckSlider.Text:SetPoint("CENTER", -5, 17)
    UpdateRangeCheckSliderText()
    
    Healium_RangeCheckSlider:SetScript("OnValueChanged",function()
        Healium.RangeCheckPeriod = 1.0 / Healium_RangeCheckSlider:GetValue()
		UpdateRangeCheckSliderText()
    end)
	
	-- ShowBuffs check
	Healium_ShowBuffsCheck = CreateFrame("CheckButton","$parentShowBuffsCheckButton",scrollchild,"OptionsCheckButtonTemplate")
    Healium_ShowBuffsCheck:SetPoint("TOPLEFT",Healium_RangeCheckCheck, "BOTTOMLEFT", 0, 0)
    
    Healium_ShowBuffsCheck.Text = Healium_ShowBuffsCheck:CreateFontString(nil, "BACKGROUND","GameFontNormal")
    Healium_ShowBuffsCheck.Text:SetPoint("LEFT", Healium_ShowBuffsCheck, "RIGHT", 0)
    Healium_ShowBuffsCheck.Text:SetText("Show Buffs")
    
    Healium_ShowBuffsCheck:SetScript("OnClick",function()
        Healium.ShowBuffs = Healium_ShowBuffsCheck:GetChecked() or false
    end)
	
    -- About Frame
    AboutTitle = CreateFrame("Frame","",scrollchild)
    AboutTitle:SetFrameStrata("TOOLTIP")
    AboutTitle:SetWidth(160)
    AboutTitle:SetHeight(20)
    
    AboutTitle.Text = AboutTitle:CreateFontString(nil, "BACKGROUND","GameFontNormalLarge")
    AboutTitle.Text:SetPoint("TOPLEFT",Healium_ShowBuffsCheck, "BOTTOMLEFT", 0, -30)
    AboutTitle.Text:SetText("About " .. Healium_AddonColor .. Healium_AddonName)
    
    HealBoxAboutFrame = CreateFrame("Frame","AboutHealium",scrollchild)

    HealBoxAboutFrame:SetWidth(335)
    HealBoxAboutFrame:SetHeight(80)
    HealBoxAboutFrame:SetPoint("TOPLEFT", AboutTitle.Text, "BOTTOMLEFT", 0, 0)

    HealBoxAboutFrame:SetBackdrop({bgFile = "",
    edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
    tile = true, tileSize = 16, edgeSize = 16,
    insets = { left = 4, right = 4, top = 4, bottom = 4 }})

    HealBoxAboutFrame.Text = HealBoxAboutFrame:CreateFontString(nil, "BACKGROUND","GameFontNormal")
    HealBoxAboutFrame.Text:SetWidth(330)
    HealBoxAboutFrame.Text:SetJustifyH("LEFT")
    HealBoxAboutFrame.Text:SetPoint("TOPLEFT", 7,-10)
    HealBoxAboutFrame.Text:SetText(Healium_AddonColor .. Healium_AddonName .. "|r "..AddonVersion.. " |cFFFFFFFFCreated by Engy of |cFFFFFF00Goofy Goomers Inc|cFFFFFFFF of Area 52.|n|n|cFFFFFFFFOriginally based on FB Healbox, which was created by Dourd of Argent Dawn EU.")

end

function Healium_UpdateButtonIcon(button, texture)
	if (texture) then
		button.icon:SetTexture(texture)
	else
		button.icon:SetTexture("Interface/Icons/INV_Misc_QuestionMark")
	end		
end

function Healium_UpdateButtonIcons()
	for i=1, Healium_MaxButtons, 1 do
		local texture = HealiumDropDownButtonIcon[i]
		
		for _, k in ipairs(Healium_Frames) do
			local button = k.buttons[i]
			if button then 
				Healium_UpdateButtonIcon(button, texture)
			end
		end
   end
end

function Healium_UpdateButtonSpell(button, spell, id, checkforCombat)

	local oldspell = button:GetAttribute("spell")
	local oldid = button.id
	
	if (oldspell == spell) and (oldid == id) then
		return 
	end
	
	if checkforCombat then 
		if InCombatLockdown() then
			return
		end
	end
	
	button.id = id
	button:SetAttribute("spell", spell)
end

function Healium_UpdateButtonSpells()
	for i=1, Healium_MaxButtons, 1 do
		local spell = HealiumDropDownButton[i]
		local id
		
		-- try existing ID first as an optimization		
		if Healium_ButtonIDs[i] then
			local spellName = GetSpellName(Healium_ButtonIDs[i], BOOKTYPE_SPELL )						
			if spellName == spell then
				id = Healium_ButtonIDs[i]
			end
		end
		
		-- look up in healing spells list
		if not id then
			for k=1, Healium_MaxClassSpells, 1 do
				if (spell == Healium_Spell.Name[k]) then
					id = Healium_Spell.ID[k]
					break
				end
			end
		end
		
		if not id then
			id = GetSpellID(spell) -- slow method
		end
		
		Healium_ButtonIDs[i] = id		
		
		for _,k in ipairs(Healium_Frames) do
			local button = k.buttons[i]
			if button then 
				Healium_UpdateButtonSpell(button, spell, id, true)			
			end
		end
   end
end

local function UpdateButtonVisibility(frame)
	-- Hide all buttons
	for i=1, Healium_MaxButtons, 1 do 
		local button = frame.buttons[i]
		if button then 
			button:Hide()
		end
	end

	-- Show buttons.  The buttons will not actually show up unless their nameplate are visible so it's fine to show them like this.	
	for i=1, Healium.ButtonCount, 1 do 
		local button = frame.buttons[i]	
		if button then 
			button:Show()
		end
	end
end

function Healium_UpdateButtonVisibility()
	for _,k in ipairs(Healium_Frames) do
		UpdateButtonVisibility(k)
	end
end

function Healium_UpdateButtons()
	Healium_UpdateButtonVisibility()
	Healium_UpdateButtonSpells()
	Healium_UpdateButtonIcons()
end

function MaxButtonSlider_Update(self)
  self.Text:SetText("Show |cFFFFFFFF"..self:GetValue().. "|r Buttons")
  Healium.ButtonCount = MaxButtonSlider:GetValue()
  Healium_UpdateButtonVisibility()
end

function Healium_ShowConfigPanel()
    if (InterfaceOptionsFrame:IsVisible()) then
      InterfaceOptionsFrame:Hide()
     else
	  InterfaceOptionsFrame_OpenToCategory(Healium_AddonName)
    end
end

local function CreateMiniMapButton()
  local button = CreateFrame("Button", "HealiumMiniMap", Minimap)
  button:SetFrameStrata("MEDIUM")

  button.icon = button:CreateTexture("icon","BACKGROUND")
  button.overlay = button:CreateTexture("icon","OVERLAY")
  button.icon:SetAllPoints()
  button.overlay:SetAllPoints()

  local highlight = button:CreateTexture(nil, "HIGHLIGHT")
  highlight:SetBlendMode("ADD")
  highlight:SetPoint("CENTER", button, "CENTER", -1, -1)
  highlight:SetWidth(34)
  highlight:SetHeight(34)
  
  button:SetPushedTexture("Interface/Buttons/UI-Quickslot-Depress")
  
  highlight:SetTexture("Interface/Minimap/UI-Minimap-ZoomButton-Highlight")
  
  local tex = button:CreateTexture("MinimapButtonOverlay", "OVERLAY")
  tex:SetTexture("Interface/Minimap/MiniMap-TrackingBorder")
  tex:SetPoint("TOPLEFT", button, "TOPLEFT", -8, 6)
  tex:SetWidth(54)
  tex:SetHeight(54)

  button.icon:SetTexture(MiniMapTexture)

  button:EnableMouse(1)
  button:RegisterForDrag("RightButton")
  button:RegisterForClicks("LeftButtonUp")
  button:SetHeight(18)
  button:SetWidth(18)
  
  button:SetPoint("TOPLEFT","Minimap","TOPLEFT",62-(80*cos(5)),(80*sin(5))-62)
  
  button:SetScript("OnEnter", function(self)
    GameTooltip:SetOwner(self, "ANCHOR_LEFT")
    GameTooltip:SetText(Healium_AddonColor .. Healium_AddonName .. "|r |n|cFF55FF55Left Mouse |cFFFFFFFF" .. Healium_AddonName .. " Menu|n|cFF55FF55Right Mouse |cFFFFFFFFMove Button|n|cFF55FF55Shift & Left Mouse |cFFFFFFFF Toggle Frames")
    GameTooltip:Show()
  end)
  
   button:SetScript("OnLeave", function(self)
    GameTooltip:Hide()
  end)
  
  -- [ Lua Only Approach For Making Dragable Frames (With Right Mouse Only) ]
  button:SetMovable(true)
  button:EnableMouse(true)
  local OnMouseDown = function(self) if(IsMouseButtonDown("RightButton")) then self:StartMoving() end end
  button:SetScript("OnMouseDown", OnMouseDown)
  button:SetScript("OnMouseUp",function(self) self:StopMovingOrSizing() end)

  button:SetScript("OnClick",function(self)  -------------------------
  
  if not (IsShiftKeyDown()) then
	ToggleDropDownMenu(1, nil, HealiumMenu, self, 0, 0)
  end
    
  if (IsShiftKeyDown()) then
	Healium_ToggleAllFrames()
  end
    
  end) ------------------------------------------------------------------
 
  return button
  
end

function Healium_RangeCheckButton(button)
    if (button.id) then
        local isUsable, noMana = IsUsableSpell(button.id, BOOKTYPE_SPELL)
          
        if isUsable then
      	 button.icon:SetVertexColor(1.0, 1.0, 1.0)
      	elseif noMana then
      	 button.icon:SetVertexColor(0.5, 0.5, 1.0)
      	else
      	  button.icon:SetVertexColor(0.3, 0.3, 0.3)
      	end
      	
       	local inRange = IsSpellInRange(button.id, BOOKTYPE_SPELL, button:GetParent().TargetUnit)
      		
		if SpellHasRange(button.id, BOOKTYPE_SPELL)  then
			if (inRange == 0) or (inRange == nil) then
				button.icon:SetVertexColor(1.0, 0.3, 0.3)
			end
		end
	end
end

function DumpVar(varName, Value)
	if (Value == nil) then 
		Healium_Print(varName .. " = (nil)")	
	else
		Healium_Print(varName .. " = " .. tostring(Value))	
	end
end

-- handles the /hb slash commands
function Healium_SlashCmdHandler(cmd)
	if (cmd == "reset") then 
--		Healium_UpdateNames()
		Healium_UpdateButtons()
		Healium_Print(Healium_AddonName .. " Reset")  
	elseif (cmd == "debug") then
		Debug = not Debug
		if (Debug) then
			Healium_Print(Healium_AddonName .. " Debug is ON")
		else
			Healium_Print(Healium_AddonName .. " Debug is OFF")
		end
	elseif (cmd == "dump") then
		DumpVar("Healium.ButtonCount",  Healium.ButtonCount)
		DumpVar("Healium.Scale", Healium.Scale)
		DumpVar("Healium.DoRangeChecks", Healium.DoRangeChecks)
		DumpVar("Healium.RangeCheckPeriod", Healium.RangeCheckPeriod)
		DumpVar("Healium_ButtonIDs", Healium_ButtonIDs)
	elseif (cmd == "cd") then
		CooldownFrame_SetTimer(HealiumParty1[1].cooldown, GetTime(), 10, 1)
	else
		Healium_Print(Healium_AddonName .. " Commands")  
		Healium_Print(Slash .. " - Shows " .. Healium_AddonName .. " commands.  (what you see here)")
		Healium_Print(Slash .. " reset - Resets the ".. Healium_AddonName .. " UI")
--		DEFAULT_CHAT_FRAME:AddMessage(Slash .. " debug - Toggles " .. Healium_AddonName .. " debugging")
--		DEFAULT_CHAT_FRAME:AddMessage(Slash .. " dump - Outputs " .. Healium_AddonName .. " variables for debugging purposes")
	end
end

function Healium_OnEvent(self, event, ...)
	local arg1 = select(1, ...)
	local arg2 = select(2, ...)

	-------------------------------------------------------------
	-- [[ Update Unit Health Display Whenever Their HP Changes ]]
	-------------------------------------------------------------
    if (event == "UNIT_HEALTH" ) then
--		if (not HealiumActive) then return 0 end
		
		if Healium_Units[arg1] then
			for i,v  in pairs(Healium_Units[arg1]) do
				Healium_UpdateUnitHealth(arg1, v)
			end
		end
	end

	if (event == "UNIT_AURA") then
		if Healium_Units[arg1] then
			for i,v  in pairs(Healium_Units[arg1]) do
				Healium_UpdateBuffs(arg1, v)
			end
		end
	end

	if event == "SPELL_UPDATE_COOLDOWN" and Healium.EnableCooldowns then
		Healium_UpdateButtonCooldowns()
	end	
		
	if event == "PLAYER_REGEN_ENABLED" then
		for i,v in ipairs(Healium_FixNameplates) do
			if (not Healium.ShowPercentage) then v.HPText:Hide() end			
			Healium_CreateButtonsForNameplate(v)
			v.name:SetWidth(80)				
			UpdateButtonVisibility(v)
		end
		
		Healium_FixButtons = {}
	end
	
	if ((event == "ADDON_LOADED") and (arg1 == Healium_AddonName)) then
		Healium_DebugPrint("ADDON_LOADED")  	
		
		-- Enable some slash commands
		SLASH_HEALIUM1 = Slash
		SlashCmdList["HEALIUM"] = Healium_SlashCmdHandler
	end
	
	if ((event == "UNIT_SPELLCAST_SENT") and ( (arg2 == "Activate Primary Spec") or (arg2 == "Activate Secondary Spec"))  ) then
--		DEFAULT_CHAT_FRAME:AddMessage("Healium Debug: Respecing Start")
		self.Respecing = true
	end

	if ( ((event == "UNIT_SPELLCAST_INTERRUPTED") or (event == "UNIT_SPELLCAST_SUCCEEDED")) and (arg1 == "player") and ( (arg2 == "Activate Primary Spec") or (arg2 == "Activate Secondary Spec"))  ) then
--		DEFAULT_CHAT_FRAME:AddMessage("Healium Debug: Respecing Interrupt or succeeded")
		self.Respecing = nil
	end
	
	if (event == "PLAYER_TALENT_UPDATE") then
		Healium_DebugPrint("PLAYER_TALENT_UPDATE")
		self.Respecing = nil
		Healium_UpdateSpells()
	end
	
	if (not self.Respecing and (event == "PLAYER_ENTERING_WORLD" or event == "SPELLS_CHANGED")) then
--		if (event == "SPELLS_CHANGED" and not HealiumActive) then return 0 end
--		if (event == "PLAYER_ENTERING_WORLD") then  DEFAULT_CHAT_FRAME:AddMessage("Healium Debug: PLAYER_ENTERING_WORLD")  end		
--		if (event == "SPELLS_CHANGED") then  DEFAULT_CHAT_FRAME:AddMessage("Healium Debug: SPELLS_CHANGED") end
		
		-- Populate the Healium_Spell Table with ID and Icon data.
		Healium_UpdateSpells()
	end
	
	if (event == "VARIABLES_LOADED") then
		Healium_DebugPrint("VARIABLES_LOADED")

		Healium_MMButton = CreateMiniMapButton()
		Healium_CreateAddonOptionFrame()
		for i=1, Healium_MaxButtons, 1 do
			UIDropDownMenu_Initialize(HealiumDropDown[i], DropDownMenu_Init)
		end
		
		Healium_InitMenu()		

        MaxButtonSlider:SetValue(Healium.ButtonCount)
        
        for i=1, Healium_MaxButtons, 1 do    
          UIDropDownMenu_SetText(HealiumDropDown[i], HealiumDropDownButton[i])
        end
		
		if (not Healium.RaidScale) then
			Healium.RaidScale = 1.0
		end
		
		if (not Healium.RangeCheckPeriod) then
			Healium.RangeCheckPeriod = DefaultRangeCheckPeriod
		end
		
		if (Healium.RangeCheckPeriod > MaxRangeCheckPeriod or Healium.RangeCheckPeriod < MinRangeCheckPeriod) then
			Healium.RangeCheckPeriod = DefaultRangeCheckPeriod
		end

		if Healium.ShowGroupFrames == nil then
			Healium.ShowGroupFrames = { }
		end
		
		if Healium.ShowToolTips == nil then 
			Healium.ShowToolTips = true
		end
		Healium_TooltipsCheck:SetChecked(Healium.ShowToolTips)		
		
		if Healium.ShowPercentage == nil then 
			Healium.ShowPercentage = true
		end
		Healium_PercentageCheck:SetChecked(Healium.ShowPercentage)
		
		if Healium.ShowPercentage == nil then 
			Healium.ShowPercentage = false
		end
		Healium_ClassColorCheck:SetChecked(Healium.UseClassColors)
		
		if Healium.ShowBuffs == nil then 
			Healium.ShowBuffs = true
		end
		Healium_ShowBuffsCheck:SetChecked(Healium.ShowBuffs)
		
		if Healium.ShowPartyFrame == nil then
			Healium.ShowPartyFrame = true
		end
		Healium_ShowPartyCheck:SetChecked(Healium.ShowPartyFrame)
		
		if Healium.ShowPetsFrame == nil then
			Healium.ShowPetsFrame = true
		end
		Healium_ShowPetsCheck:SetChecked(Healium.ShowPetsFrame)
		
		if Healium.ShowMeFrame == nil then
			Healium.ShowMeFrame = false
		end
		Healium_ShowMeCheck:SetChecked(Healium.ShowMeFrame)

		if Healium.ShowTanksFrame == nil then
			Healium.ShowTanksFrame = false
		end
		Healium_ShowTanksCheck:SetChecked(Healium.ShowTanksFrame)
		
		
		Healium_ShowGroup1Check:SetChecked(Healium.ShowGroupFrames[1])		
		Healium_ShowGroup2Check:SetChecked(Healium.ShowGroupFrames[2])				
		Healium_ShowGroup3Check:SetChecked(Healium.ShowGroupFrames[3])				
		Healium_ShowGroup4Check:SetChecked(Healium.ShowGroupFrames[4])				
		Healium_ShowGroup5Check:SetChecked(Healium.ShowGroupFrames[5])				
		Healium_ShowGroup6Check:SetChecked(Healium.ShowGroupFrames[6])				
		Healium_ShowGroup7Check:SetChecked(Healium.ShowGroupFrames[7])				
		Healium_ShowGroup8Check:SetChecked(Healium.ShowGroupFrames[8])				
		
		Healium_ScaleSlider:SetValue(Healium.Scale)
		Healium_RangeCheckCheck:SetChecked(Healium.DoRangeChecks)
		Healium_RangeCheckSlider:SetValue(1.0/Healium.RangeCheckPeriod)
		Healium_EnableCooldownsCheck:SetChecked(Healium.EnableCooldowns)
		Healium_VarsLoaded = true

		Healium_CreateUnitFrames()
		
		Healium_SetScale()		
		UpdatePercentageVisibility()		
		UpdateClassColors()
	

		Healium_ShowHidePartyFrame()
		Healium_ShowHidePetsFrame()
		Healium_ShowHideMeFrame()
		Healium_ShowHideTanksFrame()
		
		for i=1, 8, 1 do
			Healium_ShowHideGroupFrame(i)
		end
		
		Healium_UpdateButtons()		

	end
	
end


--Healium_DebugPrint("End of Healium.lua")